@@ -112,7 +112,7 @@ meta_compositor_get_window_pixmap (MetaCompositor *compositor,
 #ifdef HAVE_COMPOSITE_EXTENSIONS
   if (compositor && compositor->get_window_pixmap)
     return compositor->get_window_pixmap (compositor, window);
-  else 
+  else
     return None;
 #else
   return None;
@@ -125,7 +125,7 @@ meta_compositor_set_active_window (MetaCompositor *compositor,
                                    MetaWindow     *window)
 {
 #ifdef HAVE_COMPOSITE_EXTENSIONS
-  if (compositor && compositor->set_active_window) 
+  if (compositor && compositor->set_active_window)
     compositor->set_active_window (compositor, screen, window);
 #endif
 }
@@ -134,14 +134,14 @@ meta_compositor_set_active_window (MetaCompositor *compositor,
 void meta_compositor_begin_move (MetaCompositor *compositor,
                                  MetaWindow     *window,
                                  MetaRectangle  *initial,
-                                 int             grab_x, 
+                                 int             grab_x,
                                  int             grab_y)
 {
 }
 
 void meta_compositor_update_move (MetaCompositor *compositor,
                                   MetaWindow     *window,
-                                  int             x, 
+                                  int             x,
                                   int             y)
 {
 }
@@ -155,3 +155,23 @@ void meta_compositor_free_window (MetaCompositor *compositor,
                                   MetaWindow     *window)
 {
 }
+
+void
+meta_compositor_maximize_window (MetaCompositor *compositor,
+                                 MetaWindow     *window)
+{
+#ifdef HAVE_COMPOSITE_EXTENSIONS
+  if (compositor && compositor->maximize_window)
+    compositor->maximize_window (compositor, window);
+#endif
+}
+
+void
+meta_compositor_unmaximize_window (MetaCompositor *compositor,
+                                   MetaWindow     *window)
+{
+#ifdef HAVE_COMPOSITE_EXTENSIONS
+  if (compositor && compositor->unmaximize_window)
+    compositor->unmaximize_window (compositor, window);
+#endif
+}
